unit uFacade;

interface

type
  { Faade }
  TFacade = class
  public
    // operao do Faade
    procedure CalcularValorDeVenda(const Fidelidade: integer; const Preco: real);
  end;

implementation

uses
  SysUtils, uSubsystemCotacaoDolar, uSubsystemCalculoLoja, uSubsystemHistorico;

{ TFacade }

procedure TFacade.CalcularValorDeVenda(const Fidelidade: integer;
  const Preco: real);
var
  SubsystemCotacaoDolar: TSubsystemCotacaoDolar;
  SubsystemCalculoLoja: TSubsystemCalculoLoja;
  SubsystemHistorico: TSubsystemHistorico;
  CotacaoDolar: real;
  ValorVenda: real;
begin
  // cria as instncias dos Subsystems
  SubsystemCotacaoDolar := TSubsystemCotacaoDolar.Create;
  SubsystemCalculoLoja := TSubsystemCalculoLoja.Create;
  SubsystemHistorico := TSubsystemHistorico.Create;
  try
    // utiliza o primeiro Subsystem para consultar a cotao do Dlar
    CotacaoDolar := SubsystemCotacaoDolar.ConsultarCotacaoDolar;

    // converte em Reais
    ValorVenda := Preco * CotacaoDolar;

    // utiliza o segundo Subsytem para aplicar desconto e margem de venda
    ValorVenda := SubsystemCalculoLoja.CalcularPrecoFinal(Fidelidade, ValorVenda);

    // utiliza o terceiro Subsystem para armazenar o histrico do clculo
    SubsystemHistorico.RegistrarHistoricoDoCalculo(Fidelidade, CotacaoDolar, Preco, ValorVenda);
  finally
    // libera as instncias da memria
    FreeAndNil(SubsystemCotacaoDolar);
    FreeAndNil(SubsystemCalculoLoja);
    FreeAndNil(SubsystemHistorico);
  end;
end;

end.
